@@ -147,50 +147,50 @@ def model_arch_general(n_bins, n_outputs, kernel_coeff = 0.0005, bias_coeff = 0.
             write_to_console(short_model_summary)
     return model
 
-def model_arch_general_compnp(n_bins, n_outputs, lr=0.001, verbose=1, write_to_console=None):
-    """
-    Very simple and straight forward Neural Network with few hyperparameters
-    straighforward RELU activation strategy with cross entropy to identify the HKL
-    Tried BatchNormalization --> no significant impact
-    Tried weighted approach --> not better for HCP
-    Trying Regularaization 
-    l2(0.001) means that every coefficient in the weight matrix of the layer 
-    will add 0.001 * weight_coefficient_value**2 to the total loss of the network
-    """
-    if n_outputs >= n_bins:
-        param = n_bins
-        if param*15 < (2*n_outputs): ## quick hack; make Proper implementation
-            param = (n_bins + n_outputs)//2
-    else:
-        param = n_outputs ## More reasonable ???
-        # param = n_outputs*2 ## More reasonable ???
-        # param = n_bins//2
+# def model_arch_general_compnp(n_bins, n_outputs, lr=0.001, verbose=1, write_to_console=None):
+#     """
+#     Very simple and straight forward Neural Network with few hyperparameters
+#     straighforward RELU activation strategy with cross entropy to identify the HKL
+#     Tried BatchNormalization --> no significant impact
+#     Tried weighted approach --> not better for HCP
+#     Trying Regularaization 
+#     l2(0.001) means that every coefficient in the weight matrix of the layer 
+#     will add 0.001 * weight_coefficient_value**2 to the total loss of the network
+#     """
+#     if n_outputs >= n_bins:
+#         param = n_bins
+#         if param*15 < (2*n_outputs): ## quick hack; make Proper implementation
+#             param = (n_bins + n_outputs)//2
+#     else:
+#         param = n_outputs ## More reasonable ???
+#         # param = n_outputs*2 ## More reasonable ???
+#         # param = n_bins//2
         
-    model = Sequential()
-    model.add(keras.Input(shape=(n_bins,)))
-    ## Hidden layer 1
-    model.add(Dense(n_bins,))
-    model.add(Activation('relu'))
-    ## Hidden layer 2
-    model.add(Dense(((param)*15 + n_bins)//2,))
-    model.add(Activation('relu'))
-    ## Hidden layer 3
-    model.add(Dense((param)*15,))
-    model.add(Activation('relu'))
-    ## Output layer 
-    model.add(Dense(n_outputs, activation='softmax'))
-    ## Compile model
-    otp = tf.keras.optimizers.Adam(learning_rate=lr)
-    model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
+#     model = Sequential()
+#     model.add(keras.Input(shape=(n_bins,)))
+#     ## Hidden layer 1
+#     model.add(Dense(n_bins,))
+#     model.add(Activation('relu'))
+#     ## Hidden layer 2
+#     model.add(Dense(((param)*15 + n_bins)//2,))
+#     model.add(Activation('relu'))
+#     ## Hidden layer 3
+#     model.add(Dense((param)*15,))
+#     model.add(Activation('relu'))
+#     ## Output layer 
+#     model.add(Dense(n_outputs, activation='softmax'))
+#     ## Compile model
+#     otp = tf.keras.optimizers.Adam(learning_rate=lr)
+#     model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
 
-    if verbose == 1:
-        model.summary()
-        stringlist = []
-        model.summary(print_fn=lambda x: stringlist.append(x))
-        short_model_summary = "\n".join(stringlist)
-        if write_to_console!=None:
-            write_to_console(short_model_summary)
-    return model
+#     if verbose == 1:
+#         model.summary()
+#         stringlist = []
+#         model.summary(print_fn=lambda x: stringlist.append(x))
+#         short_model_summary = "\n".join(stringlist)
+#         if write_to_console!=None:
+#             write_to_console(short_model_summary)
+#     return model
 
 def model_arch_CNN_DNN_optimized(shape, 
                                  layer_activation="relu", 
@@ -205,7 +205,9 @@ def model_arch_CNN_DNN_optimized(shape,
                                  DNN_filters = [1000,500,100],
                                  output_neurons = 11,
                                  learning_rate = 0.001,
-                                 output="DNN"):            
+                                 output="DNN", 
+                                 write_to_console=None, 
+                                 verbose=1):            
     inputs = keras.layers.Input(shape, name="InputLayer")
     
     for lay in range(CNN_layers):
@@ -248,7 +250,14 @@ def model_arch_CNN_DNN_optimized(shape,
     ## Compile model
     otp = tf.keras.optimizers.Adam(learning_rate=learning_rate)
     model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
-    model.summary()
+
+    if verbose == 1:
+        model.summary()
+        stringlist = []
+        model.summary(print_fn=lambda x: stringlist.append(x))
+        short_model_summary = "\n".join(stringlist)
+        if write_to_console!=None:
+            write_to_console(short_model_summary)
     return model
 
 def model_arch_general_optimized(n_bins, n_outputs, kernel_coeff = 0.0005, bias_coeff = 0.0005, lr=None, verbose=1,
@@ -310,6 +319,7 @@ def model_arch_general_onelayer(n_bins, n_outputs,
                                 bias_coeff = 0.0005, lr=None, verbose=1,
                                 write_to_console=None):
     """
+    model_arch_general_onelayer
     Very simple and straight forward Neural Network with few hyperparameters
     straighforward RELU activation strategy with cross entropy to identify the HKL
     Tried BatchNormalization --> no significant impact
@@ -364,7 +374,7 @@ def predict_DNN(x, wb, temp_key):
     # Second layer
     layer1 = np.dot(layer0, wb[temp_key[3]]) + wb[temp_key[2]] 
     layer1 = np.maximum(0, layer1)
-    # Third layer ## ERROR OCCURS
+    # Third layer
     layer2 = np.dot(layer1, wb[temp_key[5]]) + wb[temp_key[4]]
     layer2 = np.maximum(0, layer2)
     # Output layer
@@ -571,6 +581,52 @@ def predict_CNN_DNN(x, wb, temp_key):
     x = softmax_forward_pass(x)
     return x
 
+
+# =============================================================================
+# USER DEFINED MODEL
+# =============================================================================
+def user_defined_model(n_bins, n_outputs, 
+                                kernel_coeff = 0.0005, 
+                                bias_coeff = 0.0005, lr=None, verbose=1,
+                                write_to_console=None):
+    """
+    model_arch_general_onelayer as user defined mode
+    
+    Please add any different architecture that you would like to experiment
+    
+    Dont forget also to update the prediction for it with numpy functions
+    to use the multiprocessing. otherwise Keras implementation of predictions
+    will lock the cpu and will not yield any results
+    
+    To see some implementation with numpy please check the predict_DNN function
+    """
+    model = Sequential()
+    model.add(keras.Input(shape=(n_bins,)))
+    ## Hidden layer 1
+    model.add(Dense(n_bins, kernel_regularizer=l2(kernel_coeff), bias_regularizer=l2(bias_coeff)))
+    # model.add(BatchNormalization())
+    model.add(Activation('relu'))
+    model.add(Dropout(0.3)) ## Adding dropout as we introduce some uncertain data with noise
+    ## Output layer 
+    model.add(Dense(n_outputs, activation='softmax'))
+    ## Compile model
+    if lr != None:
+        otp = tf.keras.optimizers.Adam(learning_rate=lr)
+        model.compile(loss='categorical_crossentropy', optimizer=otp, metrics=[metricsNN])
+    else:
+        model.compile(loss='categorical_crossentropy', optimizer="adam", metrics=[metricsNN])
+    
+    if verbose == 1:
+        model.summary()
+        stringlist = []
+        model.summary(print_fn=lambda x: stringlist.append(x))
+        short_model_summary = "\n".join(stringlist)
+        if write_to_console!=None:
+            write_to_console(short_model_summary)
+    return model
+
+
+
 # if __name__ == "__main__":
 #     pass
     # # test of numpy prediction
